require 'record'

class Record
  module Searchable
    attr_accessor :id

    module ClassMethods
      attr_accessor :records

      # Returns all records.
      # @return [<Record>]
      def all
        @records ||= []
        @records.dup
      end

      # Creates a record.
      # @return [Record]
      def create(attributes = {}, &block)
        record = new(attributes, &block)
        @records ||= []
        @records << record
        record
      end

      # Returns the specified record or records.
      # @return [<Record>, Record, nil]
      def find(*args)
        @records ||= []

        case
        when args[0].is_a?(Array)
          records = args[0].map { |id| @records.select { |record| record.id == id.to_i } }
          records.flatten!

        when args.length > 1
          records = args.map { |id| @records.select { |record| record.id == id.to_i } }
          records.flatten!

        else
          records = @records.select { |record| record.id == args[0].to_i }
          records.first
        end
      end

      # Returns the specified first number of records sorted by id.
      # @return [<Record>, Record, nil]
      def first(*args)
        @records ||= []
        records = @records.sort_by { |record| record.id }
        records.first(*args)
      end

      # Returns the specified last number of records sorted by id.
      # @return [<Record>, Record, nil]
      def last(*args)
        @records ||= []
        records = @records.sort_by { |record| record.id }
        records.last(*args)
      end
    end

    class << self
      def included(base)
        base.__send__(:extend, ClassMethods)
      end
    end

    def initialize(attributes = {})
      attributes.each { |key, value| instance_variable_set("@#{key}", value) if respond_to?("#{key}=") }
      yield self if block_given?
    end

    # Destroys self.
    # @return [void]
    def destroy
      self.class.records ||= []
      self.class.records.delete(self)
    end

    # Saves self.
    # @return [void]
    def save
      self.class.records ||= []
      self.class.records << self
    end
  end
end
