require 'record'

describe Record::Searchable do
  before(:each) do
    Record.records = []
    Record.records << Record.new(:id => 1)
    Record.records << Record.new(:id => 2)
    Record.records << Record.new(:id => 3)
  end

  let(:records) { Record.records }
  let(:records_sorted) { records.sort_by { |record| record.id } }

  describe '::all' do
    it 'returns all records' do
      expect(Record.all).to match_array(records)
    end
  end

  describe '::create' do
    it 'creates a record' do
      record = Record.create(:id => 4)
      expect(record).to be_instance_of(Record)
      expect(record).to eq(records.last)
    end
  end

  describe '::find' do
    context 'with an array argument' do
      it 'returns the specified record or records' do
        expect(Record.find([1, 2, 3])).to match_array(records.first(3))
      end
    end

    context 'with more than one argument' do
      it 'returns the specified record or records' do
        expect(Record.find(1, 2, 3)).to match_array(records.first(3))
      end
    end

    context 'with one argument' do
      it 'returns the specified record' do
        expect(Record.find(1)).to eq(records.first)
      end
    end
  end

  describe '::first' do
    context 'with an integer argument' do
      it 'returns the specified first number of records sorted by id' do
        expect(Record.first(3)).to match_array(records_sorted.first(3))
      end
    end

    context 'with no arguments' do
      it 'returns the first record sorted by id' do
        expect(Record.first).to eq(records_sorted.first)
      end
    end
  end

  describe '::last' do
    context 'with an integer argument' do
      it 'returns the specified last number of records sorted by id' do
        expect(Record.last(3)).to match_array(records_sorted.last(3))
      end
    end

    context 'with no arguments' do
      it 'returns the last record sorted by id' do
        expect(Record.last).to eq(records_sorted.last)
      end
    end
  end

  describe '#destroy' do
    it 'destroys self' do
      record = Record.new(:id => 4)
      records << record
      record.destroy
      expect(records).to_not include(record)
    end
  end

  describe '#save' do
    it 'saves self' do
      record = Record.new(:id => 4)
      record.save
      expect(records).to include(record)
    end
  end
end
